keys = {}

keys.infos = {}

keys.start_timer = Timer()
keys.refresh_key_info_timer = Timer()
keys.refresh_ui_timer = Timer()

keys.ui = {}

keys.ui.copier = {}
keys.ui.copier.selected_key_members = nil
keys.ui.copier.selected_key_info = nil

keys.ui.stations = {}

keys.ui.manager = {}

dofile('ui-stations.lua')
dofile('ui-manager.lua')
dofile('ui-copier.lua')
dofile('ui.lua')

function keys.remove_color(s)
	if s ~= nil then
		return string.gsub(s, "^.......", "")
	else
		return nil
	end
end

function keys.print(msg)
	print("\127ffffffKeys: "..msg)
end

function keys.trigger_key_info_refresh()
	keys.keys_refreshed = 0
	keys.key_count = GetNumKeysInKeychain()
	for i=1, keys.key_count do 
		local ki = {}
		ki.id, ki.desc, ki.owner_key_id, ki.time, ki.u1, ki.u2, ki.active = GetKeyInfo(i)
		-- trigger key refresh
		Keychain.list(ki.id) 
	end
end

function keys.update_ui_after_give_keys()
	
	-- copier
	while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list
	keys.ui.copier.selected_key_members = {}
	if keys.ui.copier.selected_key_info ~= nil then
		for _,ki in pairs(keys.infos) do
			if ki.id == keys.ui.copier.selected_key_info.id then
				local key_member_list_i = 1
				for _,m in pairs(ki.members) do
					table.insert(keys.ui.copier.selected_key_members, m)
					keys.ui.copier.key_member_list[key_member_list_i] = m.print_name
					key_member_list_i = key_member_list_i + 1
				end
			end
		end
	end
	keys.ui.copier.activate_buttons()

  -- manager
	while(keys.ui.manager.key_member_list[1]) do keys.ui.manager.key_member_list[1] = nil end -- clear list
	keys.ui.manager.selected_key_members = {}
	if keys.ui.manager.selected_key_info ~= nil then
		for _,ki in pairs(keys.infos) do
			if ki.id == keys.ui.manager.selected_key_info.id then
				local key_member_list_i = 1
				for _,m in pairs(ki.members) do
					table.insert(keys.ui.manager.selected_key_members, m)
					keys.ui.manager.key_member_list[key_member_list_i] = m.print_name
					key_member_list_i = key_member_list_i + 1
				end
			end
		end
	end

end

function keys.store_refreshed_key_infos()
	keys.infos = {}

	local owner_key_ids = {}

	for i=1, keys.key_count do 
		local ki = {}
		ki.id, ki.desc, ki.owner_key_id, ki.time, ki.u1, ki.u2, ki.active = GetKeyInfo(i)
		-- insert user names
		ki.members = {}
		if ki.u2 ~= nil then
			for _,userid in pairs(ki.u2) do
				local player_name = GetPlayerName(userid)
				if string.match(player_name, "^[^\*].*") then
					local faction = GetPlayerFaction(userid) or 0
					local print_name = rgbtohex(FactionColor_RGB[faction])
					print_name = print_name .. player_name
					table.insert(ki.members, {id = userid,
																		owner_key_id = ki.owner_key_id,
																		name = player_name,
																		active = ki.active,
																		print_name = print_name
																	})
				end
			end
		end
		table.sort (ki.members, function (a, b) return string.lower(a.name) < string.lower(b.name) end )

		-- store owner key ids
		if ki.owner_key_id == false then
			owner_key_ids[ki.id] = true
		end

		-- store info
		table.insert(keys.infos, ki)
	end
	table.sort (keys.infos, function (a, b) return string.lower(a.desc) < string.lower(b.desc) end )

	-- mark keys that you own
	for _,ki in pairs(keys.infos) do
		if ki.owner_key_id == false or owner_key_ids[ki.owner_key_id] ~= nil then
			ki.is_yours = true
		else
			ki.is_yours = false
		end
	end
end

-- supress spammy like messages
local orig_chatreceiver = chatreceiver.OnEvent
function chatreceiver:OnEvent(event, data)
	if event == 'CHAT_MSG_SERVER' and (
		data.msg == 'Key info received.' 
		or
		string.match(data.msg, "User Key given to .*")
		or
		data.msg == "Error: That player already has that key."
		or
		string.match(data.msg, "User Key taken from .*") )
	then
		return
	end
	orig_chatreceiver(chatreceiver, event, data)
end

-- catch the ball
function keys:OnEvent(event, data)
	if event == "CHAT_MSG_SERVER" then
		if data.msg == 'Key info received.' then
			if keys.ui.copier.key_member_list ~= nil and keys.ui.copier.key_member_list[1] == "Reloading ..." then
				-- keys are refresing after copying from clipboard:
				-- .. wait and update UI
				keys.wait_until_all_keys_are_triggered(keys.update_ui_after_give_keys)
			end
			keys.keys_refreshed = keys.keys_refreshed + 1
		end

		if string.match(data.msg, "User Key given to .*") 
		   or
		   data.msg == "Error: That player already has that key."
		   or
		   string.match(data.msg, "User Key taken from .*") 
			 then
				 -- continue 2 secs after the last "key given"
			   keys.refresh_key_info_timer:SetTimeout(2000, 
					 function() 
						 keys.trigger_key_info_refresh()
						 keys.ui.copier.key_member_list[1] = "Reloading ..." -- this flags ui to be refreshed after all keys are triggered (in OnEvent also)
					 end
				 )
	  end

		if data.msg == "Error: You do not have that Owner Key." 
			or
			data.msg == "Error: You cannot take a User Key from someone who has an Owner Key."
			or
			data.msg == "Error: That character does not exist."
			or
			string.match(data.msg, "Error: That player .* does not exist.")
		then
			   -- continue updating ui on error
				 keys.update_ui_after_give_keys()
		end

	end
end
RegisterEvent(keys, "CHAT_MSG_SERVER")

function keys.wait_until_all_keys_are_triggered(next_job)
	if keys.keys_refreshed < keys.key_count then
		-- Call this again. There is more to trigger...
		keys.start_timer:SetTimeout(1000, function() keys.wait_until_all_keys_are_triggered(next_job) end)
	else
		keys.store_refreshed_key_infos()
		next_job()
	end
end

function keys.start_func(_, args)
	keys.print("loading, please wait")	
	keys.trigger_key_info_refresh() -- this triggers refresh for all keys
	keys.wait_until_all_keys_are_triggered(keys.ui.create_dialog) -- parameter is function ptr for "next job"
end
RegisterUserCommand("keys", keys.start_func)
